@@ -18,6 +18,7 @@
  */
 
 /***************************************************************************
+ * Copyright (C) 2017-2019 ZmartZone IAM
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -141,7 +142,7 @@ char *oidc_jwt_serialize(apr_pool_t *pool, oidc_jwt_t *jwt,
 		size_t out_len;
 		if (cjose_base64url_encode((const uint8_t *) s_payload,
 				strlen(s_payload), &out, &out_len, &cjose_err) == FALSE)
-			return FALSE;
+			return NULL;
 		cser = apr_pstrmemdup(pool, out, out_len);
 		cjose_get_dealloc()(out);
 
@@ -368,7 +369,7 @@ apr_byte_t oidc_jwk_to_json(apr_pool_t *pool, oidc_jwk_t *jwk, char **s_json,
 /*
  * hash a sequence of bytes with a specific algorithm and return the result as a base64url-encoded \0 terminated string
  */
-static apr_byte_t oidc_jose_hash_and_base64url_encode(apr_pool_t *pool,
+apr_byte_t oidc_jose_hash_and_base64url_encode(apr_pool_t *pool,
 		const char *openssl_hash_algo, const char *input, int input_len,
 		char **output) {
 	oidc_jose_error_t err;
@@ -434,14 +435,14 @@ oidc_jwk_t *oidc_jwk_create_symmetric_key(apr_pool_t *pool, const char *skid,
 	if (cjose_jwk == NULL) {
 		oidc_jose_error(err, "cjose_jwk_create_oct_spec failed: %s",
 				oidc_cjose_e2s(pool, cjose_err));
-		return FALSE;
+		return NULL;
 	}
 
 	if (set_kid == TRUE) {
 		if (oidc_jwk_set_or_generate_kid(pool, cjose_jwk, skid,
 				(const char *) key, key_len, err) == FALSE) {
 			cjose_jwk_release(cjose_jwk);
-			return FALSE;
+			return NULL;
 		}
 	}
 
@@ -778,6 +779,7 @@ apr_byte_t oidc_jwt_parse(apr_pool_t *pool, const char *input_json,
 			&jwt->payload, err) == FALSE) {
 		oidc_jwt_destroy(jwt);
 		*j_jwt = NULL;
+		return FALSE;
 	}
 
 	return TRUE;
@@ -838,7 +840,7 @@ apr_byte_t oidc_jwt_sign(apr_pool_t *pool, oidc_jwt_t *jwt, oidc_jwk_t *jwk,
 	return TRUE;
 }
 
-#if (OPENSSL_VERSION_NUMBER < 0x10100000)
+#if (OPENSSL_VERSION_NUMBER < 0x10100000) || defined(LIBRESSL_VERSION_NUMBER)
 EVP_MD_CTX * EVP_MD_CTX_new() {
 	return malloc(sizeof(EVP_MD_CTX));
 }
