@@ -18,6 +18,7 @@
  */
 
 /***************************************************************************
+ * Copyright (C) 2017-2019 ZmartZone IAM
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -63,67 +64,71 @@
 
 extern module AP_MODULE_DECLARE_DATA auth_openidc_module;
 
-#define OIDC_METADATA_SUFFIX_PROVIDER                       "provider"
-#define OIDC_METADATA_SUFFIX_CLIENT                         "client"
-#define OIDC_METADATA_SUFFIX_CONF                           "conf"
+#define OIDC_METADATA_SUFFIX_PROVIDER                              "provider"
+#define OIDC_METADATA_SUFFIX_CLIENT                                "client"
+#define OIDC_METADATA_SUFFIX_CONF                                  "conf"
 
-#define OIDC_METADATA_ISSUER                                "issuer"
-#define OIDC_METADATA_RESPONSE_TYPES_SUPPORTED              "response_types_supported"
-#define OIDC_METADATA_RESPONSE_MODES_SUPPORTED              "response_modes_supported"
-#define OIDC_METADATA_AUTHORIZATION_ENDPOINT                "authorization_endpoint"
-#define OIDC_METADATA_TOKEN_ENDPOINT                        "token_endpoint"
-#define OIDC_METADATA_USERINFO_ENDPOINT                     "userinfo_endpoint"
-#define OIDC_METADATA_JWKS_URI                              "jwks_uri"
-#define OIDC_METADATA_TOKEN_ENDPOINT_AUTH_METHODS_SUPPORTED "token_endpoint_auth_methods_supported"
-#define OIDC_METADATA_REGISTRATION_ENDPOINT                 "registration_endpoint"
-#define OIDC_METADATA_CHECK_SESSION_IFRAME                  "check_session_iframe"
-#define OIDC_METADATA_END_SESSION_ENDPOINT                  "end_session_endpoint"
+#define OIDC_METADATA_ISSUER                                       "issuer"
+#define OIDC_METADATA_RESPONSE_TYPES_SUPPORTED                     "response_types_supported"
+#define OIDC_METADATA_RESPONSE_MODES_SUPPORTED                     "response_modes_supported"
+#define OIDC_METADATA_AUTHORIZATION_ENDPOINT                       "authorization_endpoint"
+#define OIDC_METADATA_TOKEN_ENDPOINT                               "token_endpoint"
+#define OIDC_METADATA_INTROSPECTION_ENDPOINT                       "introspection_endpoint"
+#define OIDC_METADATA_USERINFO_ENDPOINT                            "userinfo_endpoint"
+#define OIDC_METADATA_JWKS_URI                                     "jwks_uri"
+#define OIDC_METADATA_TOKEN_ENDPOINT_AUTH_METHODS_SUPPORTED        "token_endpoint_auth_methods_supported"
+#define OIDC_METADATA_INTROSPECTON_NDPOINT_AUTH_METHODS_SUPPORTED  "introspection_endpoint_auth_methods_supported"
+#define OIDC_METADATA_REGISTRATION_ENDPOINT                        "registration_endpoint"
+#define OIDC_METADATA_CHECK_SESSION_IFRAME                         "check_session_iframe"
+#define OIDC_METADATA_BACKCHANNEL_LOGOUT_SUPPORTED                 "backchannel_logout_supported"
 
-#define OIDC_METADATA_CLIENT_ID                             "client_id"
-#define OIDC_METADATA_CLIENT_SECRET                         "client_secret"
-#define OIDC_METADATA_CLIENT_SECRET_EXPIRES_AT              "client_secret_expires_at"
+#define OIDC_METADATA_END_SESSION_ENDPOINT                         "end_session_endpoint"
+#define OIDC_METADATA_CLIENT_ID                                    "client_id"
+#define OIDC_METADATA_CLIENT_SECRET                                "client_secret"
+#define OIDC_METADATA_CLIENT_SECRET_EXPIRES_AT                     "client_secret_expires_at"
 
-#define OIDC_METADATA_KEYS                                  "keys"
+#define OIDC_METADATA_KEYS                                         "keys"
 
-#define OIDC_METADATA_CLIENT_JWKS_URI                       "client_jwks_uri"
-#define OIDC_METADATA_ID_TOKEN_SIGNED_RESPONSE_ALG          "id_token_signed_response_alg"
-#define OIDC_METADATA_ID_TOKEN_ENCRYPTED_RESPONSE_ALG       "id_token_encrypted_response_alg"
-#define OIDC_METADATA_ID_TOKEN_ENCRYPTED_RESPONSE_ENC       "id_token_encrypted_response_enc"
-#define OIDC_METADATA_USERINFO_SIGNED_RESPONSE_ALG          "userinfo_signed_response_alg"
-#define OIDC_METADATA_USERINFO_ENCRYPTED_RESPONSE_ALG       "userinfo_encrypted_response_alg"
-#define OIDC_METADATA_USERINFO_ENCRYPTED_RESPONSE_ENC       "userinfo_encrypted_response_enc"
+#define OIDC_METADATA_CLIENT_JWKS_URI                              "client_jwks_uri"
+#define OIDC_METADATA_ID_TOKEN_SIGNED_RESPONSE_ALG                 "id_token_signed_response_alg"
+#define OIDC_METADATA_ID_TOKEN_ENCRYPTED_RESPONSE_ALG              "id_token_encrypted_response_alg"
+#define OIDC_METADATA_ID_TOKEN_ENCRYPTED_RESPONSE_ENC              "id_token_encrypted_response_enc"
+#define OIDC_METADATA_USERINFO_SIGNED_RESPONSE_ALG                 "userinfo_signed_response_alg"
+#define OIDC_METADATA_USERINFO_ENCRYPTED_RESPONSE_ALG              "userinfo_encrypted_response_alg"
+#define OIDC_METADATA_USERINFO_ENCRYPTED_RESPONSE_ENC              "userinfo_encrypted_response_enc"
 
-#define OIDC_METADATA_CLIENT_NAME                           "client_name"
-#define OIDC_METADATA_REDIRECT_URIS                         "redirect_uris"
-#define OIDC_METADATA_RESPONSE_TYPES                        "response_types"
-#define OIDC_METADATA_TOKEN_ENDPOINT_AUTH_METHOD            "token_endpoint_auth_method"
-#define OIDC_METADATA_CONTACTS                              "contacts"
-#define OIDC_METADATA_INITIATE_LOGIN_URI                    "initiate_login_uri"
-#define OIDC_METADATA_FRONTCHANNEL_LOGOUT_URI               "frontchannel_logout_uri"
-#define OIDC_METADATA_POST_LOGOUT_REDIRECT_URIS             "post_logout_redirect_uris"
-
-#define OIDC_METADATA_SSL_VALIDATE_SERVER                   "ssl_validate_server"
-#define OIDC_METADATA_SCOPE                                 "scope"
-#define OIDC_METADATA_JWKS_REFRESH_INTERVAL                 "jwks_refresh_interval"
-#define OIDC_METADATA_IDTOKEN_IAT_SLACK                     "idtoken_iat_slack"
-#define OIDC_METADATA_SESSION_MAX_DURATION                  "session_max_duration"
-#define OIDC_METADATA_AUTH_REQUEST_PARAMS                   "auth_request_params"
-#define OIDC_METADATA_TOKEN_ENDPOINT_PARAMS                 "token_endpoint_params"
-#define OIDC_METADATA_RESPONSE_MODE                         "response_mode"
-#define OIDC_METADATA_PKCE_METHOD                           "pkce_method"
-#define OIDC_METADATA_CLIENT_CONTACT                        "client_contact"
-#define OIDC_METADATA_TOKEN_ENDPOINT_AUTH                   "token_endpoint_auth"
-#define OIDC_METADATA_REGISTRATION_TOKEN                    "registration_token"
-#define OIDC_METADATA_REGISTRATION_ENDPOINT_JSON            "registration_endpoint_json"
-#define OIDC_METADATA_RESPONSE_TYPE                         "response_type"
-#define OIDC_METADATA_USERINFO_REFRESH_INTERVAL             "userinfo_refresh_interval"
-#define OIDC_METADATA_TOKEN_ENDPOINT_TLS_CLIENT_CERT        "token_endpoint_tls_client_cert"
-#define OIDC_METADATA_TOKEN_ENDPOINT_TLS_CLIENT_KEY         "token_endpoint_tls_client_key"
-#define OIDC_METADATA_REQUEST_OBJECT                        "request_object"
-#define OIDC_METADATA_USERINFO_TOKEN_METHOD                 "userinfo_token_method"
-#define OIDC_METADATA_TOKEN_BINDING_POLICY                  "token_binding_policy"
-#define OIDC_METADATA_AUTH_REQUEST_METHOD                   "auth_request_method"
-#define OIDC_METADATA_ISSUER_SPECIFIC_REDIRECT_URI          "issuer_specific_redirect_uri"
+#define OIDC_METADATA_CLIENT_NAME                                  "client_name"
+#define OIDC_METADATA_REDIRECT_URIS                                "redirect_uris"
+#define OIDC_METADATA_RESPONSE_TYPES                               "response_types"
+#define OIDC_METADATA_TOKEN_ENDPOINT_AUTH_METHOD                   "token_endpoint_auth_method"
+#define OIDC_METADATA_CONTACTS                                     "contacts"
+#define OIDC_METADATA_INITIATE_LOGIN_URI                           "initiate_login_uri"
+#define OIDC_METADATA_FRONTCHANNEL_LOGOUT_URI                      "frontchannel_logout_uri"
+#define OIDC_METADATA_BACKCHANNEL_LOGOUT_URI                       "backchannel_logout_uri"
+#define OIDC_METADATA_POST_LOGOUT_REDIRECT_URIS                    "post_logout_redirect_uris"
+#define OIDC_METADATA_IDTOKEN_BINDING_CNF                          "id_token_token_binding_cnf"
+#define OIDC_METADATA_SSL_VALIDATE_SERVER                          "ssl_validate_server"
+#define OIDC_METADATA_SCOPE                                        "scope"
+#define OIDC_METADATA_JWKS_REFRESH_INTERVAL                        "jwks_refresh_interval"
+#define OIDC_METADATA_IDTOKEN_IAT_SLACK                            "idtoken_iat_slack"
+#define OIDC_METADATA_SESSION_MAX_DURATION                         "session_max_duration"
+#define OIDC_METADATA_AUTH_REQUEST_PARAMS                          "auth_request_params"
+#define OIDC_METADATA_TOKEN_ENDPOINT_PARAMS                        "token_endpoint_params"
+#define OIDC_METADATA_RESPONSE_MODE                                "response_mode"
+#define OIDC_METADATA_PKCE_METHOD                                  "pkce_method"
+#define OIDC_METADATA_CLIENT_CONTACT                               "client_contact"
+#define OIDC_METADATA_TOKEN_ENDPOINT_AUTH                          "token_endpoint_auth"
+#define OIDC_METADATA_REGISTRATION_TOKEN                           "registration_token"
+#define OIDC_METADATA_REGISTRATION_ENDPOINT_JSON                   "registration_endpoint_json"
+#define OIDC_METADATA_RESPONSE_TYPE                                "response_type"
+#define OIDC_METADATA_USERINFO_REFRESH_INTERVAL                    "userinfo_refresh_interval"
+#define OIDC_METADATA_TOKEN_ENDPOINT_TLS_CLIENT_CERT               "token_endpoint_tls_client_cert"
+#define OIDC_METADATA_TOKEN_ENDPOINT_TLS_CLIENT_KEY                "token_endpoint_tls_client_key"
+#define OIDC_METADATA_REQUEST_OBJECT                               "request_object"
+#define OIDC_METADATA_USERINFO_TOKEN_METHOD                        "userinfo_token_method"
+#define OIDC_METADATA_TOKEN_BINDING_POLICY                         "token_binding_policy"
+#define OIDC_METADATA_AUTH_REQUEST_METHOD                          "auth_request_method"
+#define OIDC_METADATA_ISSUER_SPECIFIC_REDIRECT_URI                 "issuer_specific_redirect_uri"
 
 /*
  * get the metadata filename for a specified issuer (cq. urlencode it)
@@ -260,7 +265,7 @@ static apr_byte_t oidc_metadata_is_valid_uri(request_rec *r, const char *type,
 /*
  * check to see if JSON provider metadata is valid
  */
-static apr_byte_t oidc_metadata_provider_is_valid(request_rec *r, oidc_cfg *cfg,
+apr_byte_t oidc_metadata_provider_is_valid(request_rec *r, oidc_cfg *cfg,
 		json_t *j_provider, const char *issuer) {
 
 	/* get the "issuer" from the provider metadata and double-check that it matches what we looked for */
@@ -556,6 +561,19 @@ static apr_byte_t oidc_metadata_client_register(request_rec *r, oidc_cfg *cfg,
 							OIDC_REDIRECT_URI_REQUEST_LOGOUT,
 							OIDC_GET_STYLE_LOGOUT_PARAM_VALUE)));
 
+	// TODO: may want to add backchannel_logout_session_required
+	json_object_set_new(data, OIDC_METADATA_BACKCHANNEL_LOGOUT_URI,
+			json_string(
+					apr_psprintf(r->pool, "%s?%s=%s",
+							oidc_get_redirect_uri(r, cfg),
+							OIDC_REDIRECT_URI_REQUEST_LOGOUT,
+							OIDC_BACKCHANNEL_STYLE_LOGOUT_PARAM_VALUE)));
+
+	if (provider->token_binding_policy > OIDC_TOKEN_BINDING_POLICY_DISABLED) {
+		json_object_set_new(data, OIDC_METADATA_IDTOKEN_BINDING_CNF,
+				json_string(OIDC_CLAIM_CNF_TBH));
+	}
+
 	if (cfg->default_slo_url != NULL) {
 		json_object_set_new(data, OIDC_METADATA_POST_LOGOUT_REDIRECT_URIS,
 				json_pack("[s]", cfg->default_slo_url));
@@ -1008,6 +1026,13 @@ apr_byte_t oidc_metadata_provider_parse(request_rec *r, oidc_cfg *cfg,
 				&provider->end_session_endpoint, NULL);
 	}
 
+	// TODO: default 0 should have a defined default that may be 1...
+	if (provider->backchannel_logout_supported == OIDC_CONFIG_POS_INT_UNSET) {
+		oidc_metadata_parse_boolean(r, j_provider,
+				OIDC_METADATA_BACKCHANNEL_LOGOUT_SUPPORTED,
+				&provider->backchannel_logout_supported, 0);
+	}
+
 	if (provider->token_endpoint_auth == NULL) {
 		if (oidc_valid_string_in_array(r->pool, j_provider,
 				OIDC_METADATA_TOKEN_ENDPOINT_AUTH_METHODS_SUPPORTED,
@@ -1024,6 +1049,48 @@ apr_byte_t oidc_metadata_provider_parse(request_rec *r, oidc_cfg *cfg,
 	return TRUE;
 }
 
+apr_byte_t oidc_oauth_metadata_provider_parse(request_rec *r, oidc_cfg *c,
+		json_t *j_provider) {
+
+	char *issuer = NULL;
+
+	/* get the "issuer" from the provider metadata */
+	oidc_json_object_get_string(r->pool, j_provider, OIDC_METADATA_ISSUER,
+			&issuer, NULL);
+
+	if (c->oauth.introspection_endpoint_url == NULL) {
+		/* get a handle to the introspection endpoint */
+		oidc_metadata_parse_url(r, OIDC_METADATA_SUFFIX_PROVIDER, issuer,
+				j_provider,
+				OIDC_METADATA_INTROSPECTION_ENDPOINT,
+				&c->oauth.introspection_endpoint_url,
+				NULL);
+	}
+
+	if (c->oauth.verify_jwks_uri == NULL) {
+		/* get a handle to the jwks_uri endpoint */
+		oidc_metadata_parse_url(r, OIDC_METADATA_SUFFIX_PROVIDER, issuer,
+				j_provider,
+				OIDC_METADATA_JWKS_URI, &c->oauth.verify_jwks_uri,
+				NULL);
+	}
+
+	if (c->oauth.introspection_endpoint_auth == NULL) {
+		if (oidc_valid_string_in_array(r->pool, j_provider,
+				OIDC_METADATA_INTROSPECTON_NDPOINT_AUTH_METHODS_SUPPORTED,
+				oidc_cfg_get_valid_endpoint_auth_function(c),
+				&c->oauth.introspection_endpoint_auth,
+				TRUE) != NULL) {
+			oidc_error(r,
+					"could not find a supported token endpoint authentication method in provider metadata (%s) for entry \"" OIDC_METADATA_INTROSPECTON_NDPOINT_AUTH_METHODS_SUPPORTED "\"",
+					issuer);
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 /*
  * get a string value from a JSON object and see if it is a valid value according to the specified validation function
  */
@@ -1269,14 +1336,8 @@ apr_byte_t oidc_metadata_client_parse(request_rec *r, oidc_cfg *cfg,
 			NULL);
 
 	if (token_endpoint_auth != NULL) {
-		if ((apr_strnatcmp(token_endpoint_auth, OIDC_PROTO_CLIENT_SECRET_POST)
-				== 0)
-				|| (apr_strnatcmp(token_endpoint_auth,
-								OIDC_PROTO_CLIENT_SECRET_BASIC) == 0)
-								|| (apr_strnatcmp(token_endpoint_auth,
-												OIDC_PROTO_CLIENT_SECRET_JWT) == 0)
-												|| (apr_strnatcmp(token_endpoint_auth,
-																OIDC_PROTO_PRIVATE_KEY_JWT) == 0)) {
+		if (oidc_cfg_get_valid_endpoint_auth_function(cfg)(r->pool,
+				token_endpoint_auth) == NULL) {
 			provider->token_endpoint_auth = apr_pstrdup(r->pool,
 					token_endpoint_auth);
 		} else {
