@@ -18,6 +18,7 @@
  */
 
 /***************************************************************************
+ * Copyright (C) 2017-2019 ZmartZone IAM
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -378,6 +379,7 @@ const char *oidc_parse_boolean(apr_pool_t *pool, const char *arg,
 #define OIDC_ENDPOINT_AUTH_CLIENT_SECRET_BASIC "client_secret_basic"
 #define OIDC_ENDPOINT_AUTH_CLIENT_SECRET_JWT   "client_secret_jwt"
 #define OIDC_ENDPOINT_AUTH_PRIVATE_KEY_JWT     "private_key_jwt"
+#define OIDC_ENDPOINT_AUTH_BEARER_ACCESS_TOKEN "bearer_access_token"
 #define OIDC_ENDPOINT_AUTH_NONE                "none"
 
 /*
@@ -390,10 +392,11 @@ static const char *oidc_valid_endpoint_auth_method_impl(apr_pool_t *pool,
 			OIDC_ENDPOINT_AUTH_CLIENT_SECRET_BASIC,
 			OIDC_ENDPOINT_AUTH_CLIENT_SECRET_JWT,
 			OIDC_ENDPOINT_AUTH_NONE,
+			OIDC_ENDPOINT_AUTH_BEARER_ACCESS_TOKEN,
 			NULL,
 			NULL };
 	if (has_private_key)
-		options[3] = OIDC_ENDPOINT_AUTH_PRIVATE_KEY_JWT;
+		options[5] = OIDC_ENDPOINT_AUTH_PRIVATE_KEY_JWT;
 
 	return oidc_valid_string_option(pool, arg, options);
 }
@@ -527,6 +530,27 @@ const char *oidc_valid_session_max_duration(apr_pool_t *pool, int v) {
 	return NULL;
 }
 
+#define OIDC_MAX_NUMBER_OF_STATE_COOKIES_MIN  0
+#define OIDC_MAX_NUMBER_OF_STATE_COOKIES_MAX  255
+
+/*
+ * check the maximum number of parallel state cookies
+ */
+const char *oidc_valid_max_number_of_state_cookies(apr_pool_t *pool, int v) {
+	if (v == 0) {
+		return NULL;
+	}
+	if (v < OIDC_MAX_NUMBER_OF_STATE_COOKIES_MIN) {
+		return apr_psprintf(pool, "maximum must not be less than %d",
+				OIDC_MAX_NUMBER_OF_STATE_COOKIES_MIN);
+	}
+	if (v > OIDC_MAX_NUMBER_OF_STATE_COOKIES_MAX) {
+		return apr_psprintf(pool, "maximum must not be greater than %d",
+				OIDC_MAX_NUMBER_OF_STATE_COOKIES_MAX);
+	}
+	return NULL;
+}
+
 /*
  * parse a session max duration value from the provided string
  */
@@ -539,8 +563,8 @@ const char *oidc_parse_session_max_duration(apr_pool_t *pool, const char *arg,
 /*
  * parse a base64 encoded binary value from the provided string
  */
-static char *oidc_parse_base64(apr_pool_t *pool, const char *input,
-		char **output, int *output_len) {
+char *oidc_parse_base64(apr_pool_t *pool, const char *input, char **output,
+		int *output_len) {
 	int len = apr_base64_decode_len(input);
 	*output = apr_palloc(pool, len);
 	*output_len = apr_base64_decode(*output, input);
@@ -759,6 +783,7 @@ const char *oidc_parse_pass_userinfo_as(apr_pool_t *pool, const char *v1,
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_POST_STR   "post"
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_QUERY_STR  "query"
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_COOKIE_STR "cookie"
+#define OIDC_OAUTH_ACCEPT_TOKEN_IN_BASIC_STR  "basic"
 
 /*
  * convert an "accept OAuth 2.0 token in" byte value to a string representation
@@ -782,6 +807,10 @@ const char *oidc_accept_oauth_token_in2str(apr_pool_t *pool, apr_byte_t v) {
 		options[i] = OIDC_OAUTH_ACCEPT_TOKEN_IN_COOKIE_STR;
 		i++;
 	}
+	if (v & OIDC_OAUTH_ACCEPT_TOKEN_IN_BASIC) {
+		options[i] = OIDC_OAUTH_ACCEPT_TOKEN_IN_BASIC_STR;
+		i++;
+	}
 	return oidc_flatten_list_options(pool, options);
 }
 
@@ -797,6 +826,8 @@ static apr_byte_t oidc_parse_oauth_accept_token_in_str2byte(const char *v) {
 		return OIDC_OAUTH_ACCEPT_TOKEN_IN_QUERY;
 	if (strstr(v, OIDC_OAUTH_ACCEPT_TOKEN_IN_COOKIE_STR) == v)
 		return OIDC_OAUTH_ACCEPT_TOKEN_IN_COOKIE;
+	if (strstr(v, OIDC_OAUTH_ACCEPT_TOKEN_IN_BASIC_STR) == v)
+		return OIDC_OAUTH_ACCEPT_TOKEN_IN_BASIC;
 	return OIDC_OAUTH_ACCEPT_TOKEN_IN_DEFAULT;
 }
 
@@ -813,6 +844,7 @@ const char *oidc_parse_accept_oauth_token_in(apr_pool_t *pool, const char *arg,
 			OIDC_OAUTH_ACCEPT_TOKEN_IN_POST_STR,
 			OIDC_OAUTH_ACCEPT_TOKEN_IN_QUERY_STR,
 			OIDC_OAUTH_ACCEPT_TOKEN_IN_COOKIE_STR,
+			OIDC_OAUTH_ACCEPT_TOKEN_IN_BASIC_STR,
 			NULL };
 	const char *rv = NULL;
 
@@ -825,8 +857,6 @@ const char *oidc_parse_accept_oauth_token_in(apr_pool_t *pool, const char *arg,
 	} else {
 		p = OIDC_OAUTH_ACCEPT_TOKEN_IN_COOKIE_NAME_DEFAULT;
 	}
-	apr_hash_set(list_options, OIDC_OAUTH_ACCEPT_TOKEN_IN_OPTION_COOKIE_NAME,
-			APR_HASH_KEY_STRING, p);
 
 	rv = oidc_valid_string_option(pool, s, options);
 	if (rv != NULL)
@@ -838,6 +868,12 @@ const char *oidc_parse_accept_oauth_token_in(apr_pool_t *pool, const char *arg,
 	else
 		*b_value |= v;
 
+	if (v == OIDC_OAUTH_ACCEPT_TOKEN_IN_COOKIE) {
+		apr_hash_set(list_options,
+				OIDC_OAUTH_ACCEPT_TOKEN_IN_OPTION_COOKIE_NAME,
+				APR_HASH_KEY_STRING, p);
+	}
+
 	return NULL;
 }
 
@@ -1207,3 +1243,39 @@ const char *oidc_parse_auth_request_method(apr_pool_t *pool, const char *arg,
 
 	return NULL;
 }
+
+/*
+ * parse the maximum number of parallel state cookies
+ */
+const char *oidc_parse_max_number_of_state_cookies(apr_pool_t *pool,
+		const char *arg1, const char *arg2, int *int_value, int *bool_value) {
+	const char *rv = NULL;
+
+	rv = oidc_parse_int_valid(pool, arg1, int_value,
+			oidc_valid_max_number_of_state_cookies);
+	if ((rv == NULL) && (arg2 != NULL))
+		rv = oidc_parse_boolean(pool, arg2, bool_value);
+	return rv;
+}
+
+#define OIDC_REFRESH_ACCESS_TOKEN_BEFORE_EXPIRY_MIN 0
+#define OIDC_REFRESH_ACCESS_TOKEN_BEFORE_EXPIRY_MAX 3600 * 24 * 365
+
+/*
+ * check the boundaries for the refresh access token expiry TTL
+ */
+const char *oidc_valid_refresh_access_token_before_expiry(apr_pool_t *pool,
+		int v) {
+	return oidc_valid_int_min_max(pool, v,
+			OIDC_REFRESH_ACCESS_TOKEN_BEFORE_EXPIRY_MIN,
+			OIDC_REFRESH_ACCESS_TOKEN_BEFORE_EXPIRY_MAX);
+}
+
+/*
+ * parse an access token expiry TTL from the provided string
+ */
+const char *oidc_parse_refresh_access_token_before_expiry(apr_pool_t *pool,
+		const char *arg, int *int_value) {
+	return oidc_parse_int_valid(pool, arg, int_value,
+			oidc_valid_refresh_access_token_before_expiry);
+}
