@@ -18,6 +18,7 @@
  */
 
 /***************************************************************************
+ * Copyright (C) 2017-2019 ZmartZone IAM
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -66,6 +67,12 @@ extern module AP_MODULE_DECLARE_DATA auth_openidc_module;
 #define OIDC_SESSION_EXPIRY_KEY                   "e"
 /* the name of the provided token binding attribute in the session */
 #define OIDC_SESSION_PROVIDED_TOKEN_BINDING_KEY   "ptb"
+/* the name of the session identifier in the session */
+#define OIDC_SESSION_SESSION_ID                   "i"
+/* the name of the sub attribute in the session */
+#define OIDC_SESSION_SUB_KEY                      "sub"
+/* the name of the sid attribute in the session */
+#define OIDC_SESSION_SID_KEY                      "sid"
 
 static apr_byte_t oidc_session_encode(request_rec *r, oidc_cfg *c,
 		oidc_session_t *z, char **s_value, apr_byte_t encrypt) {
@@ -98,6 +105,29 @@ static apr_byte_t oidc_session_decode(request_rec *r, oidc_cfg *c,
 	return TRUE;
 }
 
+/*
+ * generate a unique identifier for a session
+ */
+static void oidc_session_uuid_new(request_rec *r, oidc_session_t *z) {
+	apr_uuid_t uuid;
+	apr_uuid_get(&uuid);
+	apr_uuid_format((char *) &z->uuid, &uuid);
+}
+
+/*
+ * clear contents of a session
+ */
+static void oidc_session_clear(request_rec *r, oidc_session_t *z) {
+	strncpy(z->uuid, "", strlen(""));
+	z->remote_user = NULL;
+	// NB: don't clear sid
+	z->expiry = 0;
+	if (z->state) {
+		json_decref(z->state);
+		z->state = NULL;
+	}
+}
+
 /*
  * load the session from the cache using the cookie as the index
  */
@@ -105,6 +135,7 @@ static apr_byte_t oidc_session_load_cache(request_rec *r, oidc_session_t *z) {
 	oidc_cfg *c = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
 
+	const char *stored_uuid = NULL;
 	apr_byte_t rc = FALSE;
 
 	/* get the cookie that should be our uuid/key */
@@ -116,9 +147,30 @@ static apr_byte_t oidc_session_load_cache(request_rec *r, oidc_session_t *z) {
 		rc = oidc_cache_get_session(r, uuid, &s_json);
 		if ((rc == TRUE) && (s_json != NULL)) {
 			rc = oidc_session_decode(r, c, z, s_json, FALSE);
-			if (rc == TRUE)
+			if (rc == TRUE) {
 				strncpy(z->uuid, uuid, strlen(uuid));
+
+				/* compare the session id in the cache value so it allows  us to detect cache corruption */
+				oidc_session_get(r, z, OIDC_SESSION_SESSION_ID, &stored_uuid);
+				if ((stored_uuid == NULL)
+						|| (apr_strnatcmp(stored_uuid, uuid) != 0)) {
+					oidc_error(r,
+							"cache corruption detected: stored session id (%s) is not equal to requested session id (%s)",
+							stored_uuid, uuid);
+
+					/* delete the session cookie */
+					oidc_util_set_cookie(r, oidc_cfg_dir_cookie(r), "", 0,
+							NULL);
+					/* delete the cache entry */
+					rc = oidc_cache_set_session(r, z->uuid, NULL, 0);
+					/* clear the session */
+					oidc_session_clear(r, z);
+
+					rc = FALSE;
+				}
+			}
 		}
+
 	}
 
 	return rc;
@@ -138,9 +190,14 @@ static apr_byte_t oidc_session_save_cache(request_rec *r, oidc_session_t *z,
 
 		if (apr_strnatcmp(z->uuid, "") == 0) {
 			/* get a new uuid for this session */
-			apr_uuid_t uuid;
-			apr_uuid_get(&uuid);
-			apr_uuid_format((char *) &z->uuid, &uuid);
+			oidc_session_uuid_new(r, z);
+			/* store the session id in the cache value so it allows  us to detect cache corruption */
+			oidc_session_set(r, z, OIDC_SESSION_SESSION_ID, z->uuid);
+		}
+
+		if (z->sid != NULL) {
+			oidc_cache_set_sid(r, z->sid, z->uuid, z->expiry);
+			oidc_session_set(r, z, OIDC_SESSION_SID_KEY, z->sid);
 		}
 
 		/* store the string-encoded session in the cache; encryption depends on cache backend settings */
@@ -160,6 +217,10 @@ static apr_byte_t oidc_session_save_cache(request_rec *r, oidc_session_t *z,
 											NULL);
 
 	} else {
+
+		if (z->sid != NULL)
+			oidc_cache_set_sid(r, z->sid, NULL, 0);
+
 		/* clear the cookie */
 		oidc_util_set_cookie(r, oidc_cfg_dir_cookie(r), "", 0, NULL);
 
@@ -219,10 +280,8 @@ apr_byte_t oidc_session_load(request_rec *r, oidc_session_t **zz) {
 
 	/* allocate space for the session object and fill it */
 	oidc_session_t *z = (*zz = apr_pcalloc(r->pool, sizeof(oidc_session_t)));
-
-	strncpy(z->uuid, "", strlen(""));
-	z->remote_user = NULL;
-	z->state = NULL;
+	oidc_session_clear(r, z);
+	z->sid = NULL;
 
 	if (c->session_type == OIDC_SESSION_TYPE_SERVER_CACHE)
 		/* load the session from the cache */
@@ -234,7 +293,7 @@ apr_byte_t oidc_session_load(request_rec *r, oidc_session_t **zz) {
 		/* load the session from a self-contained cookie */
 		rc = oidc_session_load_cookie(r, c, z);
 
-	if (z->state != NULL) {
+	if ((rc == TRUE) && (z->state != NULL)) {
 
 		json_t *j_expires = json_object_get(z->state, OIDC_SESSION_EXPIRY_KEY);
 		if (j_expires)
@@ -244,8 +303,7 @@ apr_byte_t oidc_session_load(request_rec *r, oidc_session_t **zz) {
 		if (apr_time_now() > z->expiry) {
 
 			oidc_warn(r, "session restored from cache has expired");
-			oidc_session_free(r, z);
-			z->state = json_object();
+			oidc_session_clear(r, z);
 
 		} else {
 
@@ -258,17 +316,16 @@ apr_byte_t oidc_session_load(request_rec *r, oidc_session_t **zz) {
 						|| (apr_strnatcmp(env_p_tb_id, ses_p_tb_id) != 0)) {
 					oidc_error(r,
 							"the Provided Token Binding ID stored in the session doesn't match the one presented by the user agent");
-					oidc_session_free(r, z);
-					z->state = json_object();
+					oidc_session_clear(r, z);
 				}
 			}
 
 			oidc_session_get(r, z, OIDC_SESSION_REMOTE_USER_KEY,
 					&z->remote_user);
-		}
-	} else {
+			oidc_session_get(r, z, OIDC_SESSION_SID_KEY,
+					&z->sid);
 
-		z->state = json_object();
+		}
 	}
 
 	return rc;
@@ -289,13 +346,13 @@ apr_byte_t oidc_session_save(request_rec *r, oidc_session_t *z,
 		oidc_session_set(r, z, OIDC_SESSION_REMOTE_USER_KEY, z->remote_user);
 		json_object_set_new(z->state, OIDC_SESSION_EXPIRY_KEY,
 				json_integer(apr_time_sec(z->expiry)));
-	}
 
-	if ((first_time) && (p_tb_id != NULL)) {
-		oidc_debug(r,
-				"Provided Token Binding ID environment variable found; adding its value to the session state");
-		oidc_session_set(r, z, OIDC_SESSION_PROVIDED_TOKEN_BINDING_KEY,
-				p_tb_id);
+		if ((first_time) && (p_tb_id != NULL)) {
+			oidc_debug(r,
+					"Provided Token Binding ID environment variable found; adding its value to the session state");
+			oidc_session_set(r, z, OIDC_SESSION_PROVIDED_TOKEN_BINDING_KEY,
+					p_tb_id);
+		}
 	}
 
 	if (c->session_type == OIDC_SESSION_TYPE_SERVER_CACHE)
@@ -315,11 +372,7 @@ apr_byte_t oidc_session_save(request_rec *r, oidc_session_t *z,
  * free resources allocated for a session
  */
 apr_byte_t oidc_session_free(request_rec *r, oidc_session_t *z) {
-	if (z->state) {
-		json_decref(z->state);
-		z->state = NULL;
-	}
-	z->expiry = 0;
+	oidc_session_clear(r, z);
 	return TRUE;
 }
 
@@ -351,10 +404,13 @@ apr_byte_t oidc_session_set(request_rec *r, oidc_session_t *z, const char *key,
 
 	/* only set it if non-NULL, otherwise delete the entry */
 	if (value) {
+		if (z->state == NULL)
+			z->state = json_object();
 		json_object_set_new(z->state, key, json_string(value));
-	} else {
+	} else if (z->state != NULL) {
 		json_object_del(z->state, key);
 	}
+
 	return TRUE;
 }
 
