@@ -18,6 +18,7 @@
  */
 
 /***************************************************************************
+ * Copyright (C) 2017-2019 ZmartZone IAM
  * Copyright (C) 2013-2017 Ping Identity Corporation
  * All rights reserved.
  *
@@ -96,7 +97,7 @@ int oidc_base64url_encode(request_rec *r, char **dst, const char *src,
 			enc_len--;
 		if ((enc_len > 0) && (enc[enc_len - 1] == ','))
 			enc_len--;
-		if ((enc_len > 0) &&(enc[enc_len - 1] == ','))
+		if ((enc_len > 0) && (enc[enc_len - 1] == ','))
 			enc_len--;
 		enc[enc_len] = '\0';
 	}
@@ -318,7 +319,15 @@ char *oidc_util_unescape_string(const request_rec *r, const char *str) {
 		oidc_error(r, "curl_easy_init() error");
 		return NULL;
 	}
-	char *result = curl_easy_unescape(curl, str, 0, 0);
+	int counter = 0;
+	char *replaced = (char *) str;
+	while (str[counter] != '\0') {
+		if (str[counter] == '+') {
+			replaced[counter] = ' ';
+		}
+		counter++;
+	}
+	char *result = curl_easy_unescape(curl, replaced, 0, 0);
 	if (result == NULL) {
 		oidc_error(r, "curl_easy_unescape() error");
 		return NULL;
@@ -344,7 +353,7 @@ char *oidc_util_html_escape(apr_pool_t *pool, const char *s) {
 	for (i = 0; i < strlen(s); i++) {
 		for (n = 0; n < len; n++) {
 			if (s[i] == chars[n]) {
-				m = (unsigned int)strlen(replace[n]);
+				m = (unsigned int) strlen(replace[n]);
 				for (k = 0; k < m; k++)
 					r[j + k] = replace[n][k];
 				j += m;
@@ -484,10 +493,24 @@ static const char *oidc_get_current_url_base(request_rec *r) {
  * get the URL that is currently being accessed
  */
 char *oidc_get_current_url(request_rec *r) {
+	char *url = NULL, *path = NULL;
+	apr_uri_t uri;
 
-	char *url = apr_pstrcat(r->pool, oidc_get_current_url_base(r), r->uri,
-			(r->args != NULL && *r->args != '\0' ? "?" : ""), r->args,
-			NULL);
+	path = r->uri;
+
+	/* check if we're dealing with a forward proxying secenario i.e. a non-relative URL */
+	if ((path) && (path[0] != '/')) {
+		memset(&uri, 0, sizeof(apr_uri_t));
+		if (apr_uri_parse(r->pool, r->uri, &uri) == APR_SUCCESS)
+			path = apr_pstrcat(r->pool, uri.path, (r->args != NULL && *r->args != '\0' ? "?" : ""), r->args, NULL);
+		else
+			oidc_warn(r, "apr_uri_parse failed on non-relative URL: %s", r->uri);
+	} else {
+		/* make sure we retain URL-encoded characters original URL that we send the user back to */
+		path = r->unparsed_uri;
+	}
+
+	url = apr_pstrcat(r->pool, oidc_get_current_url_base(r), path, NULL);
 
 	oidc_debug(r, "current URL '%s'", url);
 
@@ -521,12 +544,13 @@ const char *oidc_get_redirect_uri_iss(request_rec *r, oidc_cfg *cfg,
 	const char *redirect_uri = oidc_get_redirect_uri(r, cfg);
 	if (provider->issuer_specific_redirect_uri != 0) {
 		redirect_uri = apr_psprintf(r->pool, "%s%s%s=%s", redirect_uri,
-				strchr(redirect_uri ? redirect_uri : "", OIDC_CHAR_QUERY) != NULL ?
-						OIDC_STR_AMP :
-						OIDC_STR_QUERY,
-						OIDC_PROTO_ISS, oidc_util_escape_string(r, provider->issuer));
-//						OIDC_PROTO_CLIENT_ID,
-//						oidc_util_escape_string(r, provider->client_id));
+				strchr(redirect_uri ? redirect_uri : "",
+						OIDC_CHAR_QUERY) != NULL ?
+								OIDC_STR_AMP :
+								OIDC_STR_QUERY,
+								OIDC_PROTO_ISS, oidc_util_escape_string(r, provider->issuer));
+		//						OIDC_PROTO_CLIENT_ID,
+		//						oidc_util_escape_string(r, provider->client_id));
 		oidc_debug(r, "determined issuer specific redirect uri: %s",
 				redirect_uri);
 	}
@@ -1337,8 +1361,8 @@ int oidc_util_html_send(request_rec *r, const char *title,
 							on_load ? apr_psprintf(r->pool, " onload=\"%s()\"", on_load) : "",
 									html_body ? html_body : "<p></p>");
 
-	return oidc_util_http_send(r, html, strlen(html), OIDC_CONTENT_TYPE_HTML,
-			status_code);
+	return oidc_util_http_send(r, html, strlen(html),
+			OIDC_CONTENT_TYPE_TEXT_HTML, status_code);
 }
 
 static char *html_error_template_contents = NULL;
@@ -1348,7 +1372,8 @@ static char *html_error_template_contents = NULL;
  * that is relative to the Apache root directory
  */
 char *oidc_util_get_full_path(apr_pool_t *pool, const char *abs_or_rel_filename) {
-	return (abs_or_rel_filename) ? ap_server_root_relative(pool, abs_or_rel_filename) : NULL;
+	return (abs_or_rel_filename) ?
+			ap_server_root_relative(pool, abs_or_rel_filename) : NULL;
 }
 
 /*
@@ -1380,7 +1405,7 @@ int oidc_util_html_send_error(request_rec *r, const char *html_template,
 							description ? description : ""));
 
 			return oidc_util_http_send(r, html, strlen(html),
-					OIDC_CONTENT_TYPE_HTML, status_code);
+					OIDC_CONTENT_TYPE_TEXT_HTML, status_code);
 		}
 	}
 
@@ -1971,8 +1996,8 @@ void oidc_util_table_add_query_encoded_params(apr_pool_t *pool,
  * create a symmetric key from a client_secret
  */
 apr_byte_t oidc_util_create_symmetric_key(request_rec *r,
-		const char *client_secret, unsigned int r_key_len, const char *hash_algo,
-		apr_byte_t set_kid, oidc_jwk_t **jwk) {
+		const char *client_secret, unsigned int r_key_len,
+		const char *hash_algo, apr_byte_t set_kid, oidc_jwk_t **jwk) {
 	oidc_jose_error_t err;
 	unsigned char *key = NULL;
 	unsigned int key_len;
@@ -2067,36 +2092,49 @@ apr_hash_t * oidc_util_merge_key_sets(apr_pool_t *pool, apr_hash_t *k1,
  *     text_original: "match 292 numbers"
  *     text_replaced: "292"
  */
+apr_byte_t oidc_util_regexp_substitute(apr_pool_t *pool, const char *input,
+		const char *regexp, const char *replace, char **output,
+		char **error_str) {
 
-apr_byte_t oidc_util_regexp_substitute(
-        apr_pool_t *pool, const char *input,
-        const char *regexp, const char *replace, char **output, char **error_str) {
+	const char *errorptr = NULL;
+	int erroffset;
+	char *substituted = NULL;
+	apr_byte_t rc = FALSE;
 
-    const char *errorptr;
-    int erroffset;
-    pcre *preg;
-    char *substituted;
+	pcre *preg = pcre_compile(regexp, 0, &errorptr, &erroffset, NULL);
+	if (preg == NULL) {
+		*error_str = apr_psprintf(pool,
+				"pattern [%s] is not a valid regular expression", regexp);
+		goto out;
+	}
 
-    preg = pcre_compile(regexp, 0, &errorptr, &erroffset, NULL);
+	if (strlen(input) >= OIDC_PCRE_MAXCAPTURE - 1) {
+		*error_str =
+				apr_psprintf(pool,
+						"string length (%d) is larger than the maximum allowed for pcre_subst (%d)",
+						(int) strlen(input), OIDC_PCRE_MAXCAPTURE - 1);
+		goto out;
+	}
 
-    if (preg == NULL) {
-        *error_str = apr_psprintf(pool, "pattern [%s] is not a valid regular expression", regexp);
-        pcre_free(preg);
-        return FALSE;
-    }
+	substituted = pcre_subst(preg, NULL, input, (int) strlen(input), 0, 0,
+			replace);
+	if (substituted == NULL) {
+		*error_str =
+				apr_psprintf(pool,
+						"unknown error could not match string [%s] using pattern [%s] and replace matches in [%s]",
+						input, regexp, replace);
+		goto out;
+	}
 
-    substituted = pcre_subst(preg, NULL, input, (int) strlen(input), 0, 0, replace);
-    if (substituted) {
-        *output = apr_pstrdup(pool, substituted);
-        pcre_free(preg);
-        pcre_free(substituted);
-        return TRUE;
-    } else {
-        *error_str = apr_psprintf(pool,"unknown error could not match string [%s] using pattern [%s] and replace matches in [%s]",
-                                  input, regexp, replace);
-        pcre_free(preg);
-    }
-    return FALSE;
+	*output = apr_pstrdup(pool, substituted);
+	rc = TRUE;
+
+	out: if (substituted)
+		pcre_free(substituted);
+	if (preg)
+		pcre_free(preg);
+
+	return rc;
 }
 
 /*
@@ -2107,22 +2145,20 @@ apr_byte_t oidc_util_regexp_substitute(
 
 apr_byte_t oidc_util_regexp_first_match(apr_pool_t *pool, const char *input,
 		const char *regexp, char **output, char **error_str) {
-	const char *errorptr;
+	const char *errorptr = NULL;
 	int erroffset;
-	pcre *preg;
+	int rc = 0;
 	int subStr[OIDC_UTIL_REGEXP_MATCH_SIZE];
-	const char *psubStrMatchStr;
-
-	preg = pcre_compile(regexp, 0, &errorptr, &erroffset, NULL);
+	const char *psubStrMatchStr = NULL;
+	apr_byte_t rv = FALSE;
 
+	pcre *preg = pcre_compile(regexp, 0, &errorptr, &erroffset, NULL);
 	if (preg == NULL) {
 		*error_str = apr_psprintf(pool,
 				"pattern [%s] is not a valid regular expression", regexp);
-		pcre_free(preg);
-		return FALSE;
+		goto out;
 	}
 
-	int rc = 0;
 	if ((rc = pcre_exec(preg, NULL, input, (int) strlen(input), 0, 0, subStr,
 			OIDC_UTIL_REGEXP_MATCH_SIZE)) < 0) {
 		switch (rc) {
@@ -2150,24 +2186,27 @@ apr_byte_t oidc_util_regexp_first_match(apr_pool_t *pool, const char *input,
 			*error_str = apr_psprintf(pool, "unknown error: %d", rc);
 			break;
 		}
-		pcre_free(preg);
-		return FALSE;
+		goto out;
 	}
 
 	if (pcre_get_substring(input, subStr, rc, OIDC_UTIL_REGEXP_MATCH_NR,
 			&(psubStrMatchStr)) <= 0) {
 		*error_str = apr_psprintf(pool, "pcre_get_substring failed (rc=%d)",
 				rc);
-		pcre_free(preg);
-		return FALSE;
+		goto out;
 	}
 
 	*output = apr_pstrdup(pool, psubStrMatchStr);
+	rv = TRUE;
 
-	pcre_free_substring(psubStrMatchStr);
-	pcre_free(preg);
+out:
 
-	return TRUE;
+	if (psubStrMatchStr)
+		pcre_free_substring(psubStrMatchStr);
+	if (preg)
+		pcre_free(preg);
+
+	return rv;
 }
 
 int oidc_util_cookie_domain_valid(const char *hostname, char *cookie_domain) {
@@ -2193,7 +2232,8 @@ static const char *oidc_util_hdr_in_get(const request_rec *r, const char *name)
 	return value;
 }
 
-static const char *oidc_util_hdr_in_get_left_most_only(const request_rec *r, const char *name, const char *separator) {
+static const char *oidc_util_hdr_in_get_left_most_only(const request_rec *r,
+		const char *name, const char *separator) {
 	char *last = NULL;
 	const char *value = oidc_util_hdr_in_get(r, name);
 	if (value)
@@ -2201,6 +2241,29 @@ static const char *oidc_util_hdr_in_get_left_most_only(const request_rec *r, con
 	return NULL;
 }
 
+static apr_byte_t oidc_util_hdr_in_contains(const request_rec *r,
+		const char *name, const char *separator, const char postfix_separator,
+		const char *needle) {
+	char *ctx = NULL, *elem = NULL;
+	const char *value = oidc_util_hdr_in_get(r, name);
+	apr_byte_t rc = FALSE;
+	if (value) {
+		elem = apr_strtok(apr_pstrdup(r->pool, value), separator, &ctx);
+		while (elem != NULL) {
+			while (*elem == OIDC_CHAR_SPACE)
+				elem++;
+			if ((strncmp(elem, needle, strlen(needle)) == 0)
+					&& ((elem[strlen(needle)] == '\0')
+							|| (elem[strlen(needle)] == postfix_separator))) {
+				rc = TRUE;
+				break;
+			}
+			elem = apr_strtok(NULL, separator, &ctx);
+		}
+	}
+	return rc;
+}
+
 static void oidc_util_hdr_table_set(const request_rec *r, apr_table_t *table,
 		const char *name, const char *value) {
 
@@ -2265,7 +2328,8 @@ const char *oidc_util_hdr_in_user_agent_get(const request_rec *r) {
 }
 
 const char *oidc_util_hdr_in_x_forwarded_for_get(const request_rec *r) {
-	return oidc_util_hdr_in_get_left_most_only(r, OIDC_HTTP_HDR_X_FORWARDED_FOR, OIDC_STR_COMMA);
+	return oidc_util_hdr_in_get_left_most_only(r, OIDC_HTTP_HDR_X_FORWARDED_FOR,
+			OIDC_STR_COMMA OIDC_STR_SPACE);
 }
 
 const char *oidc_util_hdr_in_content_type_get(const request_rec *r) {
@@ -2280,20 +2344,29 @@ const char *oidc_util_hdr_in_accept_get(const request_rec *r) {
 	return oidc_util_hdr_in_get(r, OIDC_HTTP_HDR_ACCEPT);
 }
 
+apr_byte_t oidc_util_hdr_in_accept_contains(const request_rec *r,
+		const char *needle) {
+	return oidc_util_hdr_in_contains(r, OIDC_HTTP_HDR_ACCEPT, OIDC_STR_COMMA,
+			OIDC_CHAR_SEMI_COLON, needle);
+}
+
 const char *oidc_util_hdr_in_authorization_get(const request_rec *r) {
 	return oidc_util_hdr_in_get(r, OIDC_HTTP_HDR_AUTHORIZATION);
 }
 
 const char *oidc_util_hdr_in_x_forwarded_proto_get(const request_rec *r) {
-	return oidc_util_hdr_in_get_left_most_only(r, OIDC_HTTP_HDR_X_FORWARDED_PROTO, OIDC_STR_COMMA);
+	return oidc_util_hdr_in_get_left_most_only(r,
+			OIDC_HTTP_HDR_X_FORWARDED_PROTO, OIDC_STR_COMMA OIDC_STR_SPACE);
 }
 
 const char *oidc_util_hdr_in_x_forwarded_port_get(const request_rec *r) {
-	return oidc_util_hdr_in_get_left_most_only(r, OIDC_HTTP_HDR_X_FORWARDED_PORT, OIDC_STR_COMMA);
+	return oidc_util_hdr_in_get_left_most_only(r,
+			OIDC_HTTP_HDR_X_FORWARDED_PORT, OIDC_STR_COMMA OIDC_STR_SPACE);
 }
 
 const char *oidc_util_hdr_in_x_forwarded_host_get(const request_rec *r) {
-	return oidc_util_hdr_in_get_left_most_only(r, OIDC_HTTP_HDR_X_FORWARDED_HOST, OIDC_STR_COMMA);
+	return oidc_util_hdr_in_get_left_most_only(r,
+			OIDC_HTTP_HDR_X_FORWARDED_HOST, OIDC_STR_COMMA OIDC_STR_SPACE);
 }
 
 const char *oidc_util_hdr_in_host_get(const request_rec *r) {
@@ -2314,3 +2387,169 @@ const char *oidc_util_get_provided_token_binding_id(const request_rec *r) {
 		result = apr_table_get(r->subprocess_env, OIDC_TB_CFG_PROVIDED_ENV_VAR);
 	return result;
 }
+
+const char *oidc_util_get_client_cert_fingerprint(request_rec *r) {
+	const char *fingerprint = NULL;
+
+	if (r->subprocess_env == NULL)
+		goto end;
+
+	fingerprint = apr_table_get(r->subprocess_env,
+			OIDC_TB_CFG_FINGERPRINT_ENV_VAR);
+	if (fingerprint == NULL) {
+		oidc_debug(r, "no %s environment variable found",
+				OIDC_TB_CFG_FINGERPRINT_ENV_VAR);
+		goto end;
+	}
+
+end:
+
+	return fingerprint;
+}
+
+apr_byte_t oidc_util_json_validate_cnf_tbh(request_rec *r,
+		int token_binding_policy, const char *tbh_str) {
+	const char *tbp_str = NULL;
+	char *tbp = NULL;
+	int tbp_len = -1;
+	unsigned char *tbp_hash = NULL;
+	unsigned int tbp_hash_len = -1;
+	char *tbh = NULL;
+	int tbh_len = -1;
+
+	tbp_str = oidc_util_get_provided_token_binding_id(r);
+	if (tbp_str == NULL) {
+		oidc_debug(r,
+				"no Provided Token Binding ID environment variable found");
+		goto out_err;
+	}
+
+	tbp_len = oidc_base64url_decode(r->pool, &tbp, tbp_str);
+	if (tbp_len <= 0) {
+		oidc_warn(r,
+				"Provided Token Binding ID environment variable could not be decoded");
+		goto out_err;
+	}
+
+	if (oidc_jose_hash_bytes(r->pool, OIDC_JOSE_ALG_SHA256,
+			(const unsigned char *) tbp, tbp_len, &tbp_hash, &tbp_hash_len,
+			NULL) == FALSE) {
+		oidc_warn(r,
+				"hashing Provided Token Binding ID environment variable failed");
+		goto out_err;
+	}
+
+	tbh_len = oidc_base64url_decode(r->pool, &tbh, tbh_str);
+	if (tbh_len <= 0) {
+		oidc_warn(r, "cnf[\"tbh\"] provided but it could not be decoded");
+		goto out_err;
+	}
+
+	if (tbp_hash_len != tbh_len) {
+		oidc_warn(r,
+				"hash length of provided token binding ID environment variable: %d does not match length of cnf[\"tbh\"]: %d",
+				tbp_hash_len, tbh_len);
+		goto out_err;
+	}
+
+	if (memcmp(tbp_hash, tbh, tbh_len) != 0) {
+		oidc_warn(r,
+				"hash of provided token binding ID environment variable does not match cnf[\"tbh\"]");
+		goto out_err;
+	}
+
+	oidc_debug(r,
+			"hash of provided token binding ID environment variable matches cnf[\"tbh\"]");
+
+	return TRUE;
+
+out_err:
+
+	if (token_binding_policy == OIDC_TOKEN_BINDING_POLICY_OPTIONAL)
+		return TRUE;
+	if (token_binding_policy == OIDC_TOKEN_BINDING_POLICY_ENFORCED)
+		return FALSE;
+
+	// token_binding_policy == OIDC_TOKEN_BINDING_POLICY_REQURIED
+	return (tbp_str == NULL);
+}
+
+apr_byte_t oidc_util_json_validate_cnf_x5t_s256(request_rec *r,
+		int token_binding_policy, const char *x5t_256_str) {
+	const char *fingerprint = NULL;
+
+	fingerprint = oidc_util_get_client_cert_fingerprint(r);
+	if (fingerprint == NULL) {
+		oidc_debug(r, "no certificate (fingerprint) provided");
+		goto out_err;
+	}
+
+	if (apr_strnatcmp(fingerprint, x5t_256_str) != 0) {
+		oidc_warn(r,
+				"fingerprint of provided cert (%s) does not match cnf[\"x5t#S256\"] (%s)",
+				fingerprint, x5t_256_str);
+		goto out_err;
+	}
+
+	oidc_debug(r, "fingerprint of provided cert (%s) matches cnf[\"x5t#S256\"]",
+			fingerprint);
+
+	return TRUE;
+
+	out_err:
+
+	if (token_binding_policy == OIDC_TOKEN_BINDING_POLICY_OPTIONAL)
+		return TRUE;
+	if (token_binding_policy == OIDC_TOKEN_BINDING_POLICY_ENFORCED)
+		return FALSE;
+
+	// token_binding_policy == OIDC_TOKEN_BINDING_POLICY_REQURIED
+	return (fingerprint == NULL);
+}
+
+/*
+ * validate the "cnf" claim in a JWT payload
+ */
+apr_byte_t oidc_util_json_validate_cnf(request_rec *r, json_t *jwt,
+		int token_binding_policy) {
+	char *tbh_str = NULL;
+
+	oidc_debug(r, "enter: policy=%s",
+			oidc_token_binding_policy2str(r->pool, token_binding_policy));
+
+	if (token_binding_policy == OIDC_TOKEN_BINDING_POLICY_DISABLED)
+		return TRUE;
+
+	json_t *cnf = json_object_get(jwt, OIDC_CLAIM_CNF);
+	if (cnf == NULL) {
+		oidc_debug(r, "no \"%s\" claim found in the token", OIDC_CLAIM_CNF);
+		goto out_err;
+	}
+
+	oidc_jose_get_string(r->pool, cnf, OIDC_CLAIM_CNF_TBH, FALSE, &tbh_str,
+			NULL);
+	if (tbh_str != NULL)
+		return oidc_util_json_validate_cnf_tbh(r, token_binding_policy, tbh_str);
+
+	oidc_jose_get_string(r->pool, cnf, OIDC_CLAIM_CNF_X5T_S256, FALSE, &tbh_str,
+			NULL);
+	if (tbh_str != NULL)
+		return oidc_util_json_validate_cnf_x5t_s256(r, token_binding_policy,
+				tbh_str);
+
+	oidc_debug(r,
+			" \"%s\" claim found in the token but no \"%s\" or \"%s\" key found inside",
+			OIDC_CLAIM_CNF, OIDC_CLAIM_CNF_TBH, OIDC_CLAIM_CNF_X5T_S256);
+
+out_err:
+
+	if (token_binding_policy == OIDC_TOKEN_BINDING_POLICY_OPTIONAL)
+		return TRUE;
+	if (token_binding_policy == OIDC_TOKEN_BINDING_POLICY_ENFORCED)
+		return FALSE;
+
+	// token_binding_policy == OIDC_TOKEN_BINDING_POLICY_REQUIRED
+	// TODO: we don't know which token binding the client supports, do we ?
+	return FALSE;
+}
+
